# Copyright (C) 2006-2008 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from LuciSyslog import get_logger
import RicciQueries as rq

from ricci_communicator import RicciCommunicator

from LuciDB import set_node_flag, getRicciAgent, \
	getClusterNode, getStorageNode, NodeBusy, \
	setNodeStatus, resolve_nodename, getCluster, \
	delCluster, delClusterSystem, \
	CLUSTER_NODE_NEED_AUTH

from conga_constants import CLUSTER_CONFIG, LUCI_DEBUG_MODE, \
	NODE_DELETE, NODE_FORCE_DELETE, CLUSTER_DELETE, CLUSTERLIST, \
	NODE_FENCE, NODE_JOIN_CLUSTER, NODE_LEAVE_CLUSTER, NODE_REBOOT, \
	RESOURCE_REMOVE, SERVICE_DELETE, SERVICE_RESTART, \
	SERVICE_START, SERVICE_STOP

luci_log = get_logger()

def getRicciAgentForCluster(self, req):
	from LuciZope import GetReqVars
	fvar = GetReqVars(req, [ 'clustername' ])
	clustername = fvar['clustername']
	if clustername is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug('GRAFC0: no cluster name was found')
		return None
	return getRicciAgent(self, clustername)

#
# Cluster service tasks
#

def RestartCluSvc(self, rc, fvars):
	svcname = fvars['servicename']
	cluname = fvars['clustername']

	if svcname is None or cluname is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('RestartCluSvc0: svcname: %s, cluname %s' \
				% (svcname, cluname))
		return (False, { 'errors': [ 'Both a cluster service name and the cluster name must be given' ] })

	batch_number, result = rq.restartService(rc, svcname)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('RestartCluSvc0: %s failed' % svcname)
		return (False, { 'errors': [ 'An error occurred while restarting cluster service "%s"' % svcname ] })

	try:
		set_node_flag(self, cluname, rc.hostname(),
			str(batch_number), SERVICE_RESTART,
			'Restarting service "%s"' % svcname)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('RestartCluSvc1: error setting flags for service %s for cluster %s' % (svcname, cluname))

def StartCluSvc(self, rc, fvars):
	svcname = fvars['servicename']
	cluname = fvars['clustername']
	nodename = fvars['nodename']

	if svcname is None or cluname is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('StartCluSvc0: svcname: %s, cluname %s' \
				% (svcname, cluname))
		return (False, { 'errors': [ 'Both a cluster service name and the cluster name must be given' ] })

	batch_number, result = rq.startService(rc, svcname, nodename)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('StartCluSvc0: SS(%s,%s,%s) call failed' \
				% (svcname, cluname, nodename))
		return (False, { 'errors': [ 'An error occurred while starting cluster service "%s"' % svcname ] })

	try:
		if nodename is not None:
			status_msg = 'Starting cluster service "%s" on node "%s"' \
				% (svcname, nodename)
		else:
			status_msg = 'Starting cluster service "%s"' % svcname

		set_node_flag(self, cluname, rc.hostname(),
			str(batch_number), SERVICE_START, status_msg)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('StartCluSvc1: error setting flags for service %s at node %s for cluster %s' % (svcname, nodename, cluname))

def StopCluSvc(self, rc, fvars):
	svcname = fvars['servicename']
	cluname = fvars['clustername']

	if svcname is None or cluname is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('StopCluSvc0: svcname: %s, cluname %s' \
				% (svcname, cluname))
		return (False, { 'errors': [ 'Both a cluster service name and the cluster name must be given' ] })

	batch_number, result = rq.stopService(rc, svcname)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('StopCluSvc0: stop %s failed' % svcname)
		return (False, { 'errors': [ 'An error occurred while stopping cluster service "%s"' % svcname ] })

	try:
		set_node_flag(self, cluname, rc.hostname(),
			str(batch_number), SERVICE_STOP,
			'Stopping cluster service "%s"' % svcname)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('StopCluSvc1: error setting flags for service %s for cluster %s' % (svcname, cluname))

def DeleteCluSvc(self, rc, fvars, model):
	svcname = fvars['servicename']
	cluname = fvars['clustername']

	if svcname is None or cluname is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('DeleteCluSvc0: svcname: %s, cluname %s' \
				% (svcname, cluname))
		return (False, { 'errors': [ 'Both a cluster service name and the cluster name must be given' ] })

	try:
		model.deleteService(svcname)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('DeleteCluSvc1: Unable to find a service named %s for cluster %s: %r %s' % (svcname, cluname, e, str(e)))
		return (False, { 'errors': [ 'Error removing cluster service "%s"' % svcname ]})

	ret = propagateClusterConfAsync(self, model, rc,
			SERVICE_DELETE, 'Removing service "%s"' % svcname)
	if ret[0] is False:
		return ret

def MigrateCluSvc(self, rc, fvars):
	svcname = fvars['servicename']
	cluname = fvars['clustername']
	nodename = fvars['nodename']

	if svcname is None or cluname is None or nodename is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('MigrateCluSvc0: svc: %s, clu: %s, nn: %s' \
				% (svcname, cluname, nodename))
		return (False, { 'errors': [ 'A cluster service name, the cluster name, and the target node name must be given' ] })

	batch_number, result = rq.migrateService(rc, svcname, nodename)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('MigrateCluSvc0: SS(%s,%s,%s) call failed' \
				% (svcname, cluname, nodename))
		return (False, { 'errors': [ 'Error migrating cluster service "%s" to node "%s"' % (svcname, nodename) ]})

	try:
		set_node_flag(self, cluname, rc.hostname(),
			str(batch_number), SERVICE_START,
			'Migrating service "%s" to node "%s"' % (svcname, nodename))
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('MigrateCluSvc1: error setting flags for service %s at node %s for cluster %s' % (svcname, nodename, cluname))

#
# Cluster resource-related tasks
#

def DeleteResource(self, rc, model, resname):
	errstr = 'An error occurred while attempting to delete this cluster resource'
	if resname is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('DeleteResource0: no res name')
		return (False, { 'errors': [ '%s: no resource name was given' % errstr ]})

	try:
		model.deleteResource(resname)
	except KeyError, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('DeleteResource1: no res %s: %r %s' \
				% (resname, e, str(e)))
		return (False, { 'errors': [ '%s: no resource named "%s" was found' % (errstr, resname) ]})
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('DeleteResource2: err: %s: %r %s' \
				% (resname, e, str(e)))
		return (False, { 'errors': [ '%s: unable to delete resource "%s"' % (errstr, resname) ]})

	ret = propagateClusterConfAsync(self, model, rc,
			RESOURCE_REMOVE, 'Removing cluster resource "%s"' % resname)
	if ret[0] is False:
		return ret

#
# Cluster node membership-related tasks
#

def NodeJoinCluster(self,
					rc,
					clustername,
					nodename_resolved,
					enable_services=True):

	batch_number, result = rq.nodeJoinCluster(rc, enable_services=enable_services)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NJ0: batch_number and/or result is None')
		return None

	try:
		set_node_flag(self, clustername, rc.hostname(),
			str(batch_number), NODE_JOIN_CLUSTER,
			'Node "%s" joining cluster "%s"' % (nodename_resolved, clustername))
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NJ1: failed to set flags: %r %s' \
				% (e, str(e)))
		return None
	return True

def NodeLeaveCluster(	self,
						rc,
						clustername,
						nodename_resolved,
						stop_cluster=False,
						disable_services=True):
	reported_cluname = None
	try:
		cluster_info = rc.cluster_info()
		reported_cluname = cluster_info[0] or cluster_info[1]
		if not reported_cluname:
			raise Exception, 'not a cluster member'
		if reported_cluname.lower() != clustername.lower():
			raise Exception, 'cluster mismatch: expected %s, got %s' \
								% (clustername, reported_cluname)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NCL0: no cluster name: %r %s' % (e, str(e)))
		return None

	if NodeBusy(self, clustername, nodename_resolved, rc) is not False:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug('NLC1: %s is busy, can\'t leave cluster yet.' \
				% nodename_resolved)
		return None

	batch_number, result = rq.nodeLeaveCluster(rc,
							cluster_shutdown=stop_cluster,
							disable_services=disable_services)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NLC2: %s: batch_number or result is None' \
				% nodename_resolved)
		return None

	try:
		set_node_flag(self, clustername, rc.hostname(),
			str(batch_number), NODE_LEAVE_CLUSTER,
			'Node "%s" leaving cluster "%s"' % (nodename_resolved, clustername))
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NLC3: failed to set flags: %s: %r %s' \
				% (nodename_resolved, e, str(e)))
	return True

def NodeForceDeleteFromCluster(self, model, clustername, nodename, nodename_resolved):
	rc = getRicciAgent(self, clustername,
			exclude_names=[ nodename_resolved, nodename ], exclude_busy=True)

	if rc is None:
		rc = getRicciAgent(self, clustername,
			exclude_names=[ nodename_resolved, nodename ])

	if rc is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NFDFC0: no agent to delete node %s "%s"' \
				% (nodename_resolved, clustername))
		return None

	cluster_obj = getCluster(self, clustername)

	try:
		model.deleteNodeByName(nodename.lower())
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NFDFC1: deleteNode %s: %r %s' \
				% (nodename, e, str(e)))
		return None

	try:
		model.setModified(True)
		str_buf = str(model.exportModelAsString())
		if not str_buf:
			raise Exception, 'model string is blank'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NFDFC2: exportModelAsString: %r %s' \
				% (e, str(e)))
		return None

	batch_number, result = rq.setClusterConf(rc, str_buf)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NFDFC3: batch number is None')
		return None

	try:
		ret = delClusterSystem(self, cluster_obj, nodename_resolved)
		if ret is not None:
			raise Exception, ret
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NFDFC4: error deleting %s: %r %s' \
				% (nodename_resolved, e, str(e)))

	try:
		set_node_flag(self, clustername, rc.hostname(),
			str(batch_number), NODE_FORCE_DELETE,
			'Forcing the deletion of node "%s"' % nodename)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('NFDFC5: failed to set flags: %r %s' \
				% (e, str(e)))
	return True

def NodeDeleteFromCluster(	self,
							rc,
							model,
							clustername,
							nodename,
							nodename_resolved,
							delete_cluster=False):

	cluster_obj = getCluster(self, clustername)

	# We need to get a node name other than the node
	# to be deleted, then delete the node from the cluster.conf
	# and propagate it. We will need two ricci agents for this task,
	# unless we are deleting the cluster itself.

	if delete_cluster is False:
		# Make sure we can find a second node before we hose anything.
		rc2 = getRicciAgent(self, clustername,
				exclude_names=[ nodename_resolved ], exclude_busy=True)

		if rc2 is None:
			rc2 = getRicciAgent(self, clustername,
					exclude_names=[ nodename_resolved ])

		if rc2 is None:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('ND0: unable to find ricci agent to delete %s from %s' % (nodename_resolved, clustername))
			return None

	# First, delete cluster.conf from node to be deleted.
	# next, have node leave cluster.

	batch_number, result = rq.nodeLeaveCluster(rc,
							purge=True,
							disable_services=True)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('ND5: batch_number and/or result is None')
		return None

	# Unless we're deleting the whole cluster, it is not worth
	# flagging this node in DB, as we are going to delete it
	# anyway. Now, we need to the delete node from model and send out
	# new cluster.conf

	if delete_cluster is True:
		try:
			set_node_flag(self, clustername, rc.hostname(),
				str(batch_number), CLUSTER_DELETE,
				'Deleting cluster "%s": Deleting node "%s"' \
					% (clustername, nodename_resolved))
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('ND5a: failed to set flags: %r %s' \
					% (e, str(e)))
	else:
		try:
			model.deleteNodeByName(nodename.lower())
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('ND6: deleteNode %s: %r %s' \
					% (nodename, e, str(e)))
			return None

		try:
			model.setModified(True)
			str_buf = str(model.exportModelAsString())
			if not str_buf:
				raise Exception, 'model string is blank'
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('ND7: exportModelAsString: %r %s' \
					% (e, str(e)))
				return None

		# propagate the new cluster.conf via the second node
		batch_number, result = rq.setClusterConf(rc2, str_buf)
		if batch_number is None:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('ND8: batch number is None')
			return None

	try:
		ret = delClusterSystem(self, cluster_obj, nodename_resolved)
		if ret is not None:
			raise Exception, ret
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('ND9: error deleting %s: %r %s' \
				% (nodename_resolved, e, str(e)))

	if delete_cluster:
		return True

	try:
		set_node_flag(self, clustername, rc2.hostname(),
			str(batch_number), NODE_DELETE,
			'Deleting node "%s"' % nodename)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('ND10: failed to set flags: %r %s' \
				% (e, str(e)))
	return True

#
# Cluster management-related tasks.
#

def ClusterStart(self, model):
	if model is None:
		return None

	clustername = model.getClusterName()
	nodes = model.getNodes()
	if not nodes or len(nodes) < 1:
		return None

	errors = 0
	for node in nodes:
		nodename = node.getName().strip()
		nodename_resolved = resolve_nodename(self, clustername, nodename)

		try:
			# XXX - do this in parallel on all nodes
			rc = RicciCommunicator(nodename_resolved)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('CStart: RC %s: %r %s' \
					% (nodename_resolved, e, str(e)))
			errors += 1
			continue

		if NodeJoinCluster(self, rc, clustername, nodename_resolved, enable_services=False) is None:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('CStart1: nodeJoin %s' \
					% nodename_resolved)
			errors += 1

	return errors

def ClusterStop(self, model, delete=False):
	if model is None:
		return None

	clustername = model.getClusterName()
	nodes = model.getNodes()
	if not nodes or len(nodes) < 1:
		return None

	errors = 0
	for node in nodes:
		nodename = node.getName().strip()
		nodename_resolved = resolve_nodename(self, clustername, nodename)

		try:
			# XXX - do this in parallel on all nodes
			rc = RicciCommunicator(nodename_resolved)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('CStop0: [%d] RC %s: %r %s' \
					% (delete is True, str(nodename_resolved), e, str(e)))
			errors += 1
			continue

		if delete is True:
			ret = NodeDeleteFromCluster(self, rc, model, clustername,
					nodename, nodename_resolved, delete_cluster=True)
			if ret is None:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('CStop1: [1] nodeDelete failed')
				errors += 1
		else:
			ret = NodeLeaveCluster(self, rc, clustername,
					nodename_resolved, stop_cluster=True,
					disable_services=False)
			if ret is None:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('CStop2: [0] nodeLeave %s' \
						% (nodename_resolved))
				errors += 1
	return errors

def ClusterRestart(self, model):
	snum_err = ClusterStop(self, model)
	if snum_err:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('cluRestart0: ClusterStop: %d errs' \
				% snum_err)
		return snum_err

	jnum_err = ClusterStart(self, model)
	if jnum_err:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('cluRestart1: ClusterStart: %d errs' \
				% jnum_err)
	return jnum_err

def ClusterDelete(self, model):
	try:
		clustername = model.getClusterName()
		if not clustername:
			raise Exception, 'no cluster name found'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('ClusterDelete0: %r %s' % (e, str(e)))
		return None

	num_errors = ClusterStop(self, model, delete=True)
	if num_errors > 0:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('ClusterDelete2: %s: %d errors' \
				% (clustername, num_errors))
		return None

	try:
		ret = delCluster(self, clustername)
		if ret is not None:
			raise Exception, ret
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('ClusterDelete3: %s: %r %s' \
				% (clustername, e, str(e)))
		return None
	return CLUSTERLIST

def NodeReboot(self, rc, clustername, nodename_resolved):
	batch_number, result = rq.nodeReboot(rc)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('FNR0: batch_number and/or result is None')
		return None

	try:
		set_node_flag(self, clustername, rc.hostname(),
			str(batch_number), NODE_REBOOT,
			'Node "%s" is being rebooted' % nodename_resolved)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('FNR1: failed to set flags: %r %s' \
				% (e, str(e)))
	return True

def NodeFence(self, clustername, nodename, nodename_resolved):
	rc = getRicciAgent(self, clustername,
			exclude_names=[ nodename_resolved, nodename ], exclude_busy=True)

	if rc is None:
		rc = getRicciAgent(self, clustername,
				exclude_names=[ nodename_resolved, nodename ])

	if rc is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('FNF0: no ricci to fence %s for cluster %s' \
				% (nodename, clustername))
		return None

	batch_number, result = rq.nodeFence(rc, nodename)
	if batch_number is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('FNF1: batch_number and/or result is None')
		return None

	try:
		set_node_flag(self, clustername, rc.hostname(),
			str(batch_number), NODE_FENCE,
			'Node "%s" is being fenced by node "%s"' \
				% (nodename, rc.hostname()))
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('FNF2: failed to set flags: %r %s' \
				% (e, str(e)))
	return True

def propagateClusterConfAsync(	self,
								model,
								rc=None,
								action=CLUSTER_CONFIG,
								pmsg=None):
	errors = list()
	messages = list()

	try:
		clustername = model.getClusterName()
		if not clustername:
			raise Exception, 'cluster name from model.getClusterName() is blank'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('PCC0: getClusterName: %r %s' \
				% (e, str(e)))
		errors.append('Unable to determine cluster name')
		return (False, { 'errors': errors, 'messages': messages })

	if rc is None:
		rc = getRicciAgent(self, clustername, exclude_busy=True)

	if rc is None:
		rc = getRicciAgent(self, clustername)

	if rc is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('PCC1: no ricci agent for the %s cluster' \
				% clustername)
		errors.append('Unable to contact a ricci agent for cluster "%s"' \
			% clustername)
		return (False, { 'errors': errors, 'messages': messages })

	try:
		model.setModified(True)
		conf_str = str(model.exportModelAsString()).strip()
		if not conf_str:
			raise Exception, 'The new cluster configuration is blank'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('PCC2: %r %s' % (e, str(e)))
		errors.append(repr(e))
		return (False, { 'errors': errors, 'messages': messages })

	batch_id, result = rq.setClusterConf(rc, conf_str)
	if batch_id is None or result is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('PCC3: batchid or result is None')
		errors.append('Unable to propagate a new cluster configuration for %s' \
			% clustername)
		return (False, { 'errors': errors, 'messages': messages })

	if pmsg is None:
		pmsg = 'Updating the cluster configuration for "%s"' % clustername

	try:
		set_node_flag(self, clustername, rc.hostname(),
			batch_id, action, pmsg)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('PCC4: set_node_flag: %r %s' \
				% (e, str(e)))

	return (True, { 'errors': errors, 'messages': messages, 'batchid': batch_id })

def GetSystemLogs(self, fvars):
	nodename = fvars['nodename']
	if nodename is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GSL0: no node name')
		return 'No system name was given'

	clustername = fvars['clustername']
	if clustername is None:
		nodename_resolved = nodename
	else:
		nodename_resolved = resolve_nodename(self, clustername, nodename)

	try:
		rc = RicciCommunicator(nodename_resolved)
		if not rc:
			raise Exception, 'no rc'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GSL1: unexpected exception for %s: %r %s' \
				% (nodename_resolved, e, str(e)))
		return 'Ricci error while getting logs for %s' % nodename_resolved

	if not rc.authed():
		try:
			snode = getStorageNode(self, nodename_resolved)
			setNodeStatus(snode, CLUSTER_NODE_NEED_AUTH)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GSL2: %s: %r %s' \
					% (nodename_resolved, e, str(e)))

		if clustername is not None:
			try:
				cnode = getClusterNode(self, nodename_resolved, clustername)
				setNodeStatus(cnode, CLUSTER_NODE_NEED_AUTH)
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('GNL5: %s: %r %s' \
						% (nodename_resolved, e, str(e)))
		return 'Luci is not authenticated to %s. Reauthenticate first.' \
			% nodename

	return rq.getNodeLogs(rc)
